# GEO 4910 - GIS Programming
# Census Challenge
# Created by Scott Hall, Annie Cerminara, and Ademola Adeyemo
# Created from 11/4/2016 to 11/21/2016


# import csv and arcpy modules
import csv
import arcpy

#Set up to over write output
from arcpy import env
arcpy.env.overwriteOutput = True

arcpy.AddMessage("Setting up files")
#Set up files

user_fields = arcpy.GetParameterAsText(0)
input_census = arcpy.GetParameterAsText(1)
#create input shapefile
shapefile =  arcpy.GetParameterAsText(2)
output_shapefile = arcpy.GetParameterAsText(3)

output_census = (arcpy.env.scratchFolder + "\output_table.csv")

#This section reads the input Census data and the user fields, and provides a
#new table that consists of only the fields that are desired with the new field names
#Created by Scott Hall

arcpy.AddMessage("Opening census data")
# open raw census csv file and assign it to the variable raw_data
with open(input_census,"rb") as raw_data:
    # read csv file and assign it to the variable base_data
    base_data = csv.reader(raw_data)
    # read only the first row in the table
    row1=next(base_data)
    # open the desired fields csv file and assign it to the variable input_fields
    with open(user_fields,"rb") as input_fields:
        #read the csv file and assign it the variable desired_fields
        desired_fields = csv.reader(input_fields)
        #convert the base_data csv row to a list
        list_data = list(row1)
        #initialize our base_data list and assign the first 2 indexes to the list
        # this is done to make sure GEO.id2 is transfered to the output
        base_list=[1]
        # initialize the field_names list and assign the Geo.id to the list
        field_names = [list_data[1]]
        # set up loop to iterate through the desired_fields csv file
        for variable in desired_fields:
            #create a list from the current row in the desired_fields in csv
            variable_list=list(variable)
            # append the field_names list with user field names
            field_names.append(variable_list[1])
            # set up loop to iterate through the list of base_data
            for i in range(len(list_data)):
                # set up condition to check if user input field id is the same as the current base_data name
                if variable_list[0] == list_data[i]:
                    # if two variables are the same, append the base_list (consisting of indexes) with current index
                    base_list.append(i)
        # Validate field names
        #initialize list
        fields = []
        #loop through all field_names, validate and append field list
        for name in field_names:
            name = arcpy.ValidateFieldName(name)
            fields.append(name)
        # open the output csv file to write data to
        with open(output_census,"wb") as result:
            # create variable table to write to the csv output file
            table=csv.writer(result)
            # write the field_names' row
            table.writerow(fields)
            # set up loop to iterate through rows in the base_data
            for row in base_data:
                # convert csv row to a list
                raw_list_data=list(row)
                #initialize empty list
                row_list=[]
                # set up loop to iteratle through list of desired indexes(base_list)
                for i in base_list:
                    #append list with desired index of base_data row
                    row_list.append(raw_list_data[i])
                # write the row to the csv output file
                table.writerow(row_list)


# This section of the code converts a text field to a number field in the shapefile
# this is done because the field converted is to be used as the join field in the shapefile
# and joins can only be performed on number fields
# Created by Ademola Adeyemo


arcpy.AddMessage("Copying shape file")


#create a copy of original shapefile
inShape = arcpy.CopyFeatures_management(shapefile, output_shapefile)

arcpy.AddMessage("Creating new join field")
# the current text field to be converted
curGEOID = "GEOID"
# the new number field to be created
newGEOID = "GEOID_NUM"
#add new field to Census tracts to convert current GEOID to a number
arcpy.AddField_management(inShape,newGEOID,"DOUBLE")
#Set up cursor to populate newGEOID Field
with arcpy.da.UpdateCursor(inShape,[newGEOID,curGEOID]) as cursor:
#loop through cursor
    for row in cursor:
        #Convert the string "curGEOID" to a number
        row[0] = float(row[1])
        # update row
        cursor.updateRow(row)



#This section joins the shapefile and csv file
#Created by Annie cerminara

#Setup variable to join shapefile and table
joinfield = fields[0]

arcpy.AddMessage("Table to table")

#convert over the *.csv output file to a dBase table named 'TEMP.dbf'
new_table = arcpy.TableToTable_conversion(output_census,arcpy.env.scratchFolder,'TEMP.dbf')

# add indices to join tables
arcpy.AddMessage("adding index to input shapefile...")
try:
    arcpy.AddIndex_management(inShape,newGEOID,"joinindex")
except Exception as err:
    arcpy.AddMessage(err.args[0])
arcpy.AddMessage("adding index to newly created dbf table...")
try:
    arcpy.AddIndex_management(new_table,joinfield, "#")
except Exception as err:
    arcpy.AddMessage(err.args[0])

#join the shapefile and the dBase table based on the NewGEOID and first column of the dBase table
arcpy.AddMessage("Joining csv and shapefile")
arcpy.JoinField_management(inShape,newGEOID,new_table,joinfield)




